@@ -22,6 +22,7 @@
 #define __GST_CELT_DEC_H__
 
 #include <gst/gst.h>
+#include <gst/audio/gstaudiodecoder.h>
 #include <celt/celt.h>
 #include <celt/celt_header.h>
 
@@ -42,31 +43,22 @@ typedef struct _GstCeltDec GstCeltDec;
 typedef struct _GstCeltDecClass GstCeltDecClass;
 
 struct _GstCeltDec {
-  GstElement            element;
-
-  /* pads */
-  GstPad                *sinkpad;
-  GstPad                *srcpad;
+  GstAudioDecoder       element;
 
   CELTDecoder          *state;
   CELTMode             *mode;
   CELTHeader            header;
 
   gint                  frame_size;
-  GstClockTime          frame_duration;
   guint64               packetno;
 
-  GstSegment            segment;    /* STREAM LOCK */
-  gint64                granulepos; /* -1 = needs to be set from current time */
-  gboolean              discont;
-
   GstBuffer            *streamheader;
   GstBuffer            *vorbiscomment;
   GList                *extra_headers;
 };
 
 struct _GstCeltDecClass {
-  GstElementClass parent_class;
+  GstAudioDecoderClass parent_class;
 };
 
 GType gst_celt_dec_get_type (void);
