@@ -68,11 +68,6 @@ static GstStaticPadTemplate gst_jasper_dec_src_template =
         GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, Y41B, Y42B, v308 }"))
     );
 
-static void gst_jasper_dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_jasper_dec_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static void gst_jasper_dec_reset (GstJasperDec * dec);
 static GstStateChangeReturn gst_jasper_dec_change_state (GstElement * element,
     GstStateChange transition);
@@ -100,10 +95,10 @@ gst_jasper_dec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jasper_dec_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jasper_dec_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jasper_dec_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jasper_dec_sink_template);
   gst_element_class_set_details_simple (element_class,
       "Jasper JPEG2000 image decoder", "Codec/Decoder/Image",
       "Decodes JPEG2000 encoded images using jasper",
@@ -114,18 +109,13 @@ gst_jasper_dec_base_init (gpointer g_class)
 static void
 gst_jasper_dec_class_init (GstJasperDecClass * klass)
 {
-  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   GST_DEBUG_CATEGORY_INIT (gst_jasper_dec_debug, "jp2kdec", 0,
       "Jasper JPEG2000 decoder");
 
-  gobject_class->set_property = gst_jasper_dec_set_property;
-  gobject_class->get_property = gst_jasper_dec_get_property;
-
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_jasper_dec_change_state);
 }
@@ -819,36 +809,6 @@ invalid_bytes_segment:
   }
 }
 
-static void
-gst_jasper_dec_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstJasperDec *filter;
-
-  filter = GST_JASPER_DEC (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_jasper_dec_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstJasperDec *filter;
-
-  filter = GST_JASPER_DEC (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstStateChangeReturn
 gst_jasper_dec_change_state (GstElement * element, GstStateChange transition)
 {
