@@ -129,14 +129,13 @@ gst_mplex_base_init (gpointer klass)
       "High-quality MPEG/DVD/SVCD/VCD video/audio multiplexer",
       "Andrew Stevens <andrew.stevens@nexgo.de>\n"
       "Ronald Bultje <rbultje@ronald.bitfreak.net>\n"
-      "Mark Nauwelaerts <mnauw@users.sourceforge.net");
+      "Mark Nauwelaerts <mnauw@users.sourceforge.net>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_templ));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&video_sink_templ));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&audio_sink_templ));
+  gst_element_class_add_static_pad_template (element_class, &src_templ);
+  gst_element_class_add_static_pad_template (element_class,
+      &video_sink_templ);
+  gst_element_class_add_static_pad_template (element_class,
+      &audio_sink_templ);
 }
 
 static void
