@@ -268,6 +268,7 @@ rsn_dvdsrc_finalize (GObject * object)
   g_mutex_free (src->dvd_lock);
   g_mutex_free (src->branch_lock);
   g_cond_free (src->still_cond);
+  g_free (src->device);
 
   gst_buffer_replace (&src->alloc_buf, NULL);
   gst_buffer_replace (&src->next_buf, NULL);
@@ -1118,7 +1119,7 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
       break;
   }
 
-  if (src->highlight_event && have_dvd_lock) {
+  if (src->highlight_event && have_dvd_lock && src->in_playing) {
     GstEvent *hl_event = src->highlight_event;
 
     src->highlight_event = NULL;
@@ -1409,8 +1410,12 @@ rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
     }
   }
 
-  highlight_event = src->highlight_event;
-  src->highlight_event = NULL;
+  if (src->in_playing) {
+    highlight_event = src->highlight_event;
+    src->highlight_event = NULL;
+  } else {
+    highlight_event = NULL;
+  }
 
   /* Schedule a clock callback for the any pending nav packet */
   rsn_dvdsrc_check_nav_blocks (src);
