@@ -118,10 +118,8 @@ gst_camerabin_video_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (eklass,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (eklass,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (eklass, &sink_template);
+  gst_element_class_add_static_pad_template (eklass, &src_template);
   gst_element_class_set_details_simple (eklass,
       "Video capture bin for camerabin", "Bin/Video",
       "Process and store video data",
@@ -637,6 +635,13 @@ gst_camerabin_video_create_elements (GstCameraBinVideo * vid)
       G_CALLBACK (gst_camerabin_drop_eos_probe), vid);
   gst_object_unref (vid_srcpad);
 
+  /* audio source is not always present and might be set to NULL during operation */
+  if (vid->aud_src
+      && g_object_class_find_property (G_OBJECT_GET_CLASS (vid->aud_src),
+          "provide-clock")) {
+    g_object_set (vid->aud_src, "provide-clock", FALSE, NULL);
+  }
+
   GST_DEBUG ("created video elements");
 
   return TRUE;
