@@ -430,6 +430,7 @@ gst_camera_bin_start_capture (GstCameraBin2 * camerabin)
 
   g_signal_emit_by_name (camerabin->src, "start-capture", NULL);
   if (camerabin->mode == MODE_VIDEO) {
+    camerabin->audio_send_newseg = TRUE;
     if (camerabin->audio_src)
       gst_element_set_state (camerabin->audio_src, GST_STATE_PLAYING);
 
@@ -815,15 +816,13 @@ gst_camera_bin_class_init (GstCameraBin2Class * klass)
 
   /* TODO
    * Review before stable
-   * - We use a profile for video recording properties and here we have
-   *   elements for image capture. This is slightly inconsistent.
    * - One problem with using encodebin for images here is how jifmux
    *   autoplugging works. We need to give it a higher rank and fix its
    *   caps (it has image/jpeg on sink and src pads). Preliminary tests
    *   show that jifmux is picked if image/jpeg is the caps of a container
    *   profile. So this could work.
    * - There seems to be a problem with encodebin for images currently as
-   *   it autoplugs a videorate that ony starts outputing buffers after
+   *   it autoplugs a videorate that only starts outputing buffers after
    *   getting the 2nd buffer.
    */
   g_object_class_install_property (object_class, PROP_IMAGE_ENCODING_PROFILE,
@@ -1043,7 +1042,6 @@ gst_camera_bin_handle_message (GstBin * bin, GstMessage * message)
         }
         dec_counter = TRUE;
       } else if (gst_structure_has_name (structure, "preview-image")) {
-        GValue *value;
         gchar *location = NULL;
 
         g_mutex_lock (camerabin->preview_list_mutex);
@@ -1063,11 +1061,11 @@ gst_camera_bin_handle_message (GstBin * bin, GstMessage * message)
         g_mutex_unlock (camerabin->preview_list_mutex);
 
         if (location) {
-          value = g_new0 (GValue, 1);
-          g_value_init (value, G_TYPE_STRING);
-          g_value_take_string (value, location);
+          GValue value = { 0 };
+          g_value_init (&value, G_TYPE_STRING);
+          g_value_take_string (&value, location);
           gst_structure_take_value ((GstStructure *) structure, "location",
-              value);
+              &value);
         }
 
         GST_LOG_OBJECT (bin, "received preview-image message");
@@ -1089,6 +1087,8 @@ gst_camera_bin_handle_message (GstBin * bin, GstMessage * message)
         }
         dec_counter = TRUE;
       }
+      g_error_free (err);
+      g_free (debug);
     }
       break;
     case GST_MESSAGE_EOS:{
@@ -1434,18 +1434,46 @@ gst_camera_bin_image_sink_event_probe (GstPad * pad, GstEvent * event,
 }
 
 static gboolean
-gst_camera_bin_audio_src_event_probe (GstPad * pad, GstEvent * event,
+gst_camera_bin_audio_src_data_probe (GstPad * pad, GstMiniObject * obj,
     gpointer data)
 {
   GstCameraBin2 *camera = data;
   gboolean ret = TRUE;
 
-  if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
-    /* we only let an EOS pass when the user is stopping a capture */
-    if (camera->audio_drop_eos) {
+  if (GST_IS_BUFFER (obj)) {
+    if (G_UNLIKELY (camera->audio_send_newseg)) {
+      GstBuffer *buf = GST_BUFFER_CAST (obj);
+      GstClockTime ts = GST_BUFFER_TIMESTAMP (buf);
+      GstPad *peer;
+
+      if (!GST_CLOCK_TIME_IS_VALID (ts)) {
+        ts = 0;
+      }
+
+      peer = gst_pad_get_peer (pad);
+      g_return_val_if_fail (peer != NULL, TRUE);
+
+      gst_pad_send_event (peer, gst_event_new_new_segment (FALSE, 1.0,
+              GST_FORMAT_TIME, ts, -1, 0));
+
+      gst_object_unref (peer);
+
+      camera->audio_send_newseg = FALSE;
+    }
+  } else {
+    GstEvent *event = GST_EVENT_CAST (obj);
+    if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+      /* we only let an EOS pass when the user is stopping a capture */
+      if (camera->audio_drop_eos) {
+        ret = FALSE;
+      } else {
+        camera->audio_drop_eos = TRUE;
+        /* should already be false, but reinforce in case no buffers get
+         * pushed */
+        camera->audio_send_newseg = FALSE;
+      }
+    } else if (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT) {
       ret = FALSE;
-    } else {
-      camera->audio_drop_eos = TRUE;
     }
   }
 
@@ -1649,7 +1677,7 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
       NULL);
 
   if (camera->video_profile_switch) {
-    GST_DEBUG_OBJECT (camera, "Switching encodebin's profile");
+    GST_DEBUG_OBJECT (camera, "Switching video-encodebin's profile");
     g_object_set (camera->video_encodebin, "profile", camera->video_profile,
         NULL);
     if (GST_PAD_LINK_FAILED (gst_camera_bin_link_encodebin (camera,
@@ -1664,7 +1692,7 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
   }
 
   if (camera->image_profile_switch) {
-    GST_DEBUG_OBJECT (camera, "Switching encodebin's profile");
+    GST_DEBUG_OBJECT (camera, "Switching image-encodebin's profile");
     g_object_set (camera->image_encodebin, "profile", camera->image_profile,
         NULL);
     if (GST_PAD_LINK_FAILED (gst_camera_bin_link_encodebin (camera,
@@ -1801,10 +1829,11 @@ gst_camera_bin_create_elements (GstCameraBin2 * camera)
 
     srcpad = gst_element_get_static_pad (camera->audio_src, "src");
 
-    /* drop EOS for audiosrc elements that push them on state_changes
-     * (basesrc does this) */
-    gst_pad_add_event_probe (srcpad,
-        (GCallback) gst_camera_bin_audio_src_event_probe, camera);
+    /* 1) drop EOS for audiosrc elements that push them on state_changes
+     * (basesrc does this) 
+     * 2) Fix newsegment events to have start time = first buffer ts */
+    gst_pad_add_data_probe (srcpad,
+        (GCallback) gst_camera_bin_audio_src_data_probe, camera);
 
     gst_object_unref (srcpad);
   }
@@ -1862,6 +1891,7 @@ gst_camera_bin_change_state (GstElement * element, GstStateChange trans)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_CAMERA_BIN2_RESET_PROCESSING_COUNTER (camera);
       camera->audio_drop_eos = TRUE;
+      camera->audio_send_newseg = FALSE;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       if (GST_STATE (camera->videosink) >= GST_STATE_PAUSED)
@@ -1931,6 +1961,9 @@ gst_camera_bin_send_event (GstElement * element, GstEvent * event)
   GstCameraBin2 *camera = GST_CAMERA_BIN2_CAST (element);
   gboolean res;
 
+  /* avoid losing our ref to send_event */
+  gst_event_ref (event);
+
   res = GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
@@ -1956,6 +1989,7 @@ gst_camera_bin_send_event (GstElement * element, GstEvent * event)
       break;
   }
 
+  gst_event_unref (event);
   return res;
 }
 
