@@ -49,7 +49,8 @@ colorspace_convert_new (GstVideoFormat to_format, ColorSpaceColorSpec to_spec,
       || to_spec == COLOR_SPEC_RGB, NULL);
   g_return_val_if_fail (!gst_video_format_is_yuv (to_format)
       || to_spec == COLOR_SPEC_YUV_BT709
-      || to_spec == COLOR_SPEC_YUV_BT470_6, NULL);
+      || to_spec == COLOR_SPEC_YUV_BT470_6
+      || to_spec == COLOR_SPEC_YUV_JPEG, NULL);
   g_return_val_if_fail (gst_video_format_is_rgb (to_format)
       || gst_video_format_is_yuv (to_format)
       || (gst_video_format_is_gray (to_format) &&
@@ -59,7 +60,8 @@ colorspace_convert_new (GstVideoFormat to_format, ColorSpaceColorSpec to_spec,
       || from_spec == COLOR_SPEC_RGB, NULL);
   g_return_val_if_fail (!gst_video_format_is_yuv (from_format)
       || from_spec == COLOR_SPEC_YUV_BT709
-      || from_spec == COLOR_SPEC_YUV_BT470_6, NULL);
+      || from_spec == COLOR_SPEC_YUV_BT470_6
+      || from_spec == COLOR_SPEC_YUV_JPEG, NULL);
   g_return_val_if_fail (gst_video_format_is_rgb (from_format)
       || gst_video_format_is_yuv (from_format)
       || (gst_video_format_is_gray (from_format) &&
@@ -1306,7 +1308,7 @@ putline16_r210 (ColorspaceConvert * convert, guint8 * dest, const guint16 * src,
 {
   int i;
   guint8 *destline = FRAME_GET_LINE (dest, 0, j);
-  for (i = 0; i < convert->width / 2; i++) {
+  for (i = 0; i < convert->width; i++) {
     guint32 x = 0;
     x |= (src[i * 4 + 1] & 0xffc0) << 14;
     x |= (src[i * 4 + 2] & 0xffc0) << 4;
@@ -1830,7 +1832,7 @@ convert_I420_YUY2 (ColorspaceConvert * convert, guint8 * dest,
 {
   int i;
 
-  for (i = 0; i < convert->height; i += 2) {
+  for (i = 0; i < GST_ROUND_DOWN_2 (convert->height); i += 2) {
     cogorc_convert_I420_YUY2 (FRAME_GET_LINE (dest, 0, i),
         FRAME_GET_LINE (dest, 0, i + 1),
         FRAME_GET_LINE (src, 0, i),
@@ -1838,6 +1840,12 @@ convert_I420_YUY2 (ColorspaceConvert * convert, guint8 * dest,
         FRAME_GET_LINE (src, 1, i >> 1),
         FRAME_GET_LINE (src, 2, i >> 1), (convert->width + 1) / 2);
   }
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_I420 (convert, convert->tmpline, src, convert->height - 1);
+    putline_YUY2 (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -1846,7 +1854,7 @@ convert_I420_UYVY (ColorspaceConvert * convert, guint8 * dest,
 {
   int i;
 
-  for (i = 0; i < convert->height; i += 2) {
+  for (i = 0; i < GST_ROUND_DOWN_2 (convert->height); i += 2) {
     cogorc_convert_I420_UYVY (FRAME_GET_LINE (dest, 0, i),
         FRAME_GET_LINE (dest, 0, i + 1),
         FRAME_GET_LINE (src, 0, i),
@@ -1854,6 +1862,12 @@ convert_I420_UYVY (ColorspaceConvert * convert, guint8 * dest,
         FRAME_GET_LINE (src, 1, i >> 1),
         FRAME_GET_LINE (src, 2, i >> 1), (convert->width + 1) / 2);
   }
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_I420 (convert, convert->tmpline, src, convert->height - 1);
+    putline_UYVY (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -1862,7 +1876,7 @@ convert_I420_AYUV (ColorspaceConvert * convert, guint8 * dest,
 {
   int i;
 
-  for (i = 0; i < convert->height; i += 2) {
+  for (i = 0; i < GST_ROUND_DOWN_2 (convert->height); i += 2) {
     cogorc_convert_I420_AYUV (FRAME_GET_LINE (dest, 0, i),
         FRAME_GET_LINE (dest, 0, i + 1),
         FRAME_GET_LINE (src, 0, i),
@@ -1870,6 +1884,12 @@ convert_I420_AYUV (ColorspaceConvert * convert, guint8 * dest,
         FRAME_GET_LINE (src, 1, i >> 1),
         FRAME_GET_LINE (src, 2, i >> 1), convert->width);
   }
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_I420 (convert, convert->tmpline, src, convert->height - 1);
+    putline_AYUV (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -1902,23 +1922,31 @@ convert_I420_Y444 (ColorspaceConvert * convert, guint8 * dest,
   cogorc_planar_chroma_420_444 (FRAME_GET_LINE (dest, 1, 0),
       2 * convert->dest_stride[1], FRAME_GET_LINE (dest, 1, 1),
       2 * convert->dest_stride[1], FRAME_GET_LINE (src, 1, 0),
-      convert->src_stride[1], (convert->width + 1) / 2,
-      (convert->height + 1) / 2);
+      convert->src_stride[1], (convert->width + 1) / 2, convert->height / 2);
 
   cogorc_planar_chroma_420_444 (FRAME_GET_LINE (dest, 2, 0),
       2 * convert->dest_stride[2], FRAME_GET_LINE (dest, 2, 1),
       2 * convert->dest_stride[2], FRAME_GET_LINE (src, 2, 0),
-      convert->src_stride[2], (convert->width + 1) / 2,
-      (convert->height + 1) / 2);
+      convert->src_stride[2], (convert->width + 1) / 2, convert->height / 2);
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_I420 (convert, convert->tmpline, src, convert->height - 1);
+    putline_Y444 (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
 convert_YUY2_I420 (ColorspaceConvert * convert, guint8 * dest,
     const guint8 * src)
 {
-  int i;
+  int i, h;
 
-  for (i = 0; i < convert->height; i += 2) {
+  h = convert->height;
+  if (convert->width & 1)
+    h--;
+
+  for (i = 0; i < h; i += 2) {
     cogorc_convert_YUY2_I420 (FRAME_GET_LINE (dest, 0, i),
         FRAME_GET_LINE (dest, 0, i + 1),
         FRAME_GET_LINE (dest, 1, i >> 1),
@@ -1926,6 +1954,12 @@ convert_YUY2_I420 (ColorspaceConvert * convert, guint8 * dest,
         FRAME_GET_LINE (src, 0, i),
         FRAME_GET_LINE (src, 0, i + 1), (convert->width + 1) / 2);
   }
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_YUY2 (convert, convert->tmpline, src, convert->height - 1);
+    putline_I420 (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -1934,7 +1968,14 @@ convert_YUY2_AYUV (ColorspaceConvert * convert, guint8 * dest,
 {
   cogorc_convert_YUY2_AYUV (FRAME_GET_LINE (dest, 0, 0),
       convert->dest_stride[0], FRAME_GET_LINE (src, 0, 0),
-      convert->src_stride[0], (convert->width + 1) / 2, convert->height);
+      convert->src_stride[0], (convert->width + 1) / 2,
+      convert->height & 1 ? convert->height - 1 : convert->height);
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_YUY2 (convert, convert->tmpline, src, convert->height - 1);
+    putline_AYUV (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -1966,7 +2007,7 @@ convert_UYVY_I420 (ColorspaceConvert * convert, guint8 * dest,
 {
   int i;
 
-  for (i = 0; i < convert->height; i += 2) {
+  for (i = 0; i < GST_ROUND_DOWN_2 (convert->height); i += 2) {
     cogorc_convert_UYVY_I420 (FRAME_GET_LINE (dest, 0, i),
         FRAME_GET_LINE (dest, 0, i + 1),
         FRAME_GET_LINE (dest, 1, i >> 1),
@@ -1974,6 +2015,12 @@ convert_UYVY_I420 (ColorspaceConvert * convert, guint8 * dest,
         FRAME_GET_LINE (src, 0, i),
         FRAME_GET_LINE (src, 0, i + 1), (convert->width + 1) / 2);
   }
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_UYVY (convert, convert->tmpline, src, convert->height - 1);
+    putline_I420 (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -1982,7 +2029,14 @@ convert_UYVY_AYUV (ColorspaceConvert * convert, guint8 * dest,
 {
   cogorc_convert_UYVY_AYUV (FRAME_GET_LINE (dest, 0, 0),
       convert->dest_stride[0], FRAME_GET_LINE (src, 0, 0),
-      convert->src_stride[0], (convert->width + 1) / 2, convert->height);
+      convert->src_stride[0], (convert->width + 1) / 2,
+      convert->height & 1 ? convert->height - 1 : convert->height);
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_UYVY (convert, convert->tmpline, src, convert->height - 1);
+    putline_AYUV (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
@@ -2055,7 +2109,15 @@ convert_AYUV_Y42B (ColorspaceConvert * convert, guint8 * dest,
       convert->dest_stride[0], FRAME_GET_LINE (dest, 1, 0),
       convert->dest_stride[1], FRAME_GET_LINE (dest, 2, 0),
       convert->dest_stride[2], FRAME_GET_LINE (src, 0, 0),
-      convert->src_stride[0], (convert->width + 1) / 2, convert->height);
+      convert->src_stride[0], (convert->width + 1) / 2,
+      convert->height & 1 ? convert->height - 1 : convert->height);
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_AYUV (convert, convert->tmpline, src, convert->height - 1);
+    putline_Y42B (convert, dest, convert->tmpline, convert->height - 1);
+  }
+
 }
 
 static void
@@ -2081,13 +2143,20 @@ convert_Y42B_I420 (ColorspaceConvert * convert, guint8 * dest,
       convert->dest_stride[1], FRAME_GET_LINE (src, 1, 0),
       2 * convert->src_stride[1], FRAME_GET_LINE (src, 1, 1),
       2 * convert->src_stride[1], (convert->width + 1) / 2,
-      (convert->height + 1) / 2);
+      convert->height / 2);
 
   cogorc_planar_chroma_422_420 (FRAME_GET_LINE (dest, 2, 0),
       convert->dest_stride[2], FRAME_GET_LINE (src, 2, 0),
       2 * convert->src_stride[2], FRAME_GET_LINE (src, 2, 1),
       2 * convert->src_stride[2], (convert->width + 1) / 2,
-      (convert->height + 1) / 2);
+      convert->height / 2);
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_Y42B (convert, convert->tmpline, src, convert->height - 1);
+    putline_I420 (convert, dest, convert->tmpline, convert->height - 1);
+  }
+
 }
 
 static void
@@ -2152,13 +2221,19 @@ convert_Y444_I420 (ColorspaceConvert * convert, guint8 * dest,
       convert->dest_stride[1], FRAME_GET_LINE (src, 1, 0),
       2 * convert->src_stride[1], FRAME_GET_LINE (src, 1, 1),
       2 * convert->src_stride[1], (convert->width + 1) / 2,
-      (convert->height + 1) / 2);
+      convert->height / 2);
 
   cogorc_planar_chroma_444_420 (FRAME_GET_LINE (dest, 2, 0),
       convert->dest_stride[2], FRAME_GET_LINE (src, 2, 0),
       2 * convert->src_stride[2], FRAME_GET_LINE (src, 2, 1),
       2 * convert->src_stride[2], (convert->width + 1) / 2,
-      (convert->height + 1) / 2);
+      convert->height / 2);
+
+  /* now handle last line */
+  if (convert->height & 1) {
+    getline_Y444 (convert, convert->tmpline, src, convert->height - 1);
+    putline_I420 (convert, dest, convert->tmpline, convert->height - 1);
+  }
 }
 
 static void
