@@ -55,10 +55,10 @@ gst_checksum_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_checksum_sink_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_checksum_sink_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_checksum_sink_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_checksum_sink_sink_template);
 
   gst_element_class_set_details_simple (element_class, "Checksum sink",
       "Debug/Sink", "Calculates a checksum for buffers",
