@@ -137,13 +137,13 @@ gst_jif_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jif_mux_src_pad_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jif_mux_sink_pad_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jif_mux_src_pad_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jif_mux_sink_pad_template);
   gst_element_class_set_details_simple (element_class,
       "JPEG stream muxer",
-      "Video/Muxer",
+      "Video/Formatter",
       "Remuxes JPEG images with markers and tags",
       "Arnout Vandecappelle (Essensium/Mind) <arnout@mind.be>");
 }
