@@ -69,7 +69,9 @@ static GstStaticPadTemplate mpegpsmux_sink_factory =
         "video/x-dirac;"
         "video/x-h264;"
         "audio/mpeg, "
-        "mpegversion = (int) { 1, 2, 4 };"
+        "mpegversion = (int) { 1, 2 };"
+        "audio/mpeg, "
+        "mpegversion = (int) 4, stream-format = (string) { raw, adts }; "
         "audio/x-lpcm, "
         "width = (int) { 16, 20, 24 }, "
         "rate = (int) { 48000, 96000 }, "
