@@ -108,6 +108,16 @@ mpeg_util_find_start_code (guint32 * sync_word, guint8 * cur, guint8 * end)
       return cur;
     }
 
+    /* accelerate search for start code */
+    if (*cur > 1) {
+      while (cur < (end - 4) && *cur > 1)
+        if (cur[3] > 1)
+          cur += 4;
+        else
+          cur++;
+      code = 0xffffff00;
+    }
+
     /* Add the next available byte to the collected sync word */
     code |= *cur++;
   }
@@ -501,8 +511,6 @@ mpeg_util_parse_extension_packet (MPEGSeqHdr * hdr, guint8 * data, guint8 * end)
     case MPEG_PACKET_EXT_SEQUENCE:
     {
       /* Parse a Sequence Extension */
-      gboolean low_delay;
-      guint8 chroma_format;
       guint8 horiz_size_ext, vert_size_ext;
       guint8 fps_n_ext, fps_d_ext;
 
@@ -513,10 +521,10 @@ mpeg_util_parse_extension_packet (MPEGSeqHdr * hdr, guint8 * data, guint8 * end)
       hdr->profile = data[0] & 0x0f;    /* profile (0:2) + escape bit (3) */
       hdr->level = (data[1] >> 4) & 0x0f;
       hdr->progressive = data[1] & 0x08;
-      chroma_format = (data[1] >> 2) & 0x03;
+      /* chroma_format = (data[1] >> 2) & 0x03; */
       horiz_size_ext = ((data[1] << 1) & 0x02) | ((data[2] >> 7) & 0x01);
       vert_size_ext = (data[2] >> 5) & 0x03;
-      low_delay = data[5] >> 7;
+      /* low_delay = data[5] >> 7; */
       fps_n_ext = (data[5] >> 5) & 0x03;
       fps_d_ext = data[5] & 0x1f;
 
@@ -539,7 +547,6 @@ mpeg_util_parse_sequence_hdr (MPEGSeqHdr * hdr, guint8 * data, guint8 * end)
   guint32 code;
   guint8 dar_idx, fps_idx;
   guint32 sync_word = 0xffffffff;
-  gboolean constrained_flag;
   gboolean load_intra_flag;
   gboolean load_non_intra_flag;
 
@@ -574,7 +581,7 @@ mpeg_util_parse_sequence_hdr (MPEGSeqHdr * hdr, guint8 * data, guint8 * end)
     hdr->bitrate *= 400;
   }
 
-  constrained_flag = (data[7] >> 2) & 0x01;
+  /* constrained_flag = (data[7] >> 2) & 0x01; */
   load_intra_flag = (data[7] >> 1) & 0x01;
   if (load_intra_flag) {
     if (G_UNLIKELY ((end - data) < 64))
