@@ -21,41 +21,39 @@
 /**
  * SECTION:element-scaletempo
  *
- * <refsect2>
- * <para>
  * Scale tempo while maintaining pitch
  * (WSOLA-like technique with cross correlation)
  * Inspired by SoundTouch library by Olli Parviainen
- * </para>
- * <para>
+ *
  * Use Sceletempo to apply playback rates without the chipmunk effect.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
  * <para>
- * <programlisting>
+ * |[
  * filesrc location=media.ext ! decodebin name=d \
  *     d. ! queue ! audioconvert ! audioresample ! scaletempo ! audioconvert ! audioresample ! autoaudiosink \
  *     d. ! queue ! ffmpegcolorspace ! autovideosink
- * </programlisting>
+ * ]|
  * OR
- * <programlisting>
+ * |[
  * playbin uri=... audio_sink="scaletempo ! audioconvert ! audioresample ! autoaudiosink"
- * </programlisting>
+ * ]|
  * When an application sends a seek event with rate != 1.0, Scaletempo applies
  * the rate change by scaling the tempo without scaling the pitch.
- * </para>
- * <para>
- * Scaletempo works by producing audio in constant sized chunks (a "stride") but
- * consuming chunks proportional to the playback rate.
- * </para>
- * <para>
+ *
+ * Scaletempo works by producing audio in constant sized chunks
+ * (#GstScaletempo:stride) but consuming chunks proportional to the playback
+ * rate.
+ *
  * Scaletempo then smooths the output by blending the end of one stride with
- * the next ("overlap").
- * </para>
- * <para>
+ * the next (#GstScaletempo:overlap).
+ *
  * Scaletempo smooths the overlap further by searching within the input buffer
- * for the best overlap position.  Scaletempo uses a statistical cross correlation
- * (roughly a dot-product).  Scaletempo consumes most of its CPU cycles here.
+ * for the best overlap position.  Scaletempo uses a statistical cross
+ * correlation (roughly a dot-product).  Scaletempo consumes most of its CPU
+ * cycles here. One can use the #GstScaletempo:search propery to tune how far
+ * the algoritm looks.
  * </para>
  * </refsect2>
  */
@@ -684,10 +682,9 @@ gst_scaletempo_base_init (gpointer klass)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template);
   gst_element_class_set_details_simple (element_class, "Scaletempo",
       "Filter/Effect/Rate",
       "Sync audio tempo with playback rate",
