@@ -1,7 +1,7 @@
 #ifndef __CAPTURE_H__
 #define __CAPTURE_H__
 
-#include "DeckLinkAPI.h"
+#include "gstdecklink.h"
 
 class DeckLinkCaptureDelegate : public IDeckLinkInputCallback
 {
@@ -18,8 +18,8 @@ class DeckLinkCaptureDelegate : public IDeckLinkInputCallback
     void *priv;
 
   private:
-    ULONG				m_refCount;
-    pthread_mutex_t		m_mutex;
+    ULONG m_refCount;
+    GMutex *m_mutex;
 };
 
 #endif
