@@ -212,13 +212,14 @@ gst_dshowvideosink_init_interfaces (GType type)
   GST_DEBUG_CATEGORY_INIT (dshowvideosink_debug, "dshowvideosink", 0, \
       "DirectShow video sink");
 }
+
 static void
 gst_dshowvideosink_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template);
 
   gst_element_class_set_details_simple (element_class, "DirectShow video sink",
       "Sink/Video", "Display data using a DirectShow video renderer",
