@@ -196,9 +196,14 @@ gst_linsys_sdi_sink_get_property (GObject * object, guint property_id,
 void
 gst_linsys_sdi_sink_dispose (GObject * object)
 {
+  GstLinsysSdiSink *linsyssdisink;
+
   g_return_if_fail (GST_IS_LINSYS_SDI_SINK (object));
+  linsyssdisink = GST_LINSYS_SDI_SINK (object);
 
   /* clean up as possible.  may be called multiple times */
+  g_free (linsyssdisink->device);
+  linsyssdisink->device = NULL;
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
