@@ -212,9 +212,12 @@ gst_linsys_sdi_src_get_property (GObject * object, guint property_id,
 void
 gst_linsys_sdi_src_dispose (GObject * object)
 {
-  g_return_if_fail (GST_IS_LINSYS_SDI_SRC (object));
+  GstLinsysSdiSrc *linsyssdisrc = GST_LINSYS_SDI_SRC (object);
+  g_return_if_fail (linsyssdisrc != NULL);
 
   /* clean up as possible.  may be called multiple times */
+  g_free (linsyssdisrc->device);
+  linsyssdisrc->device = NULL;
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
